import _ from 'lodash';

function ownKeys(object, enumerableOnly) {
  var keys = Object.keys(object);

  if (Object.getOwnPropertySymbols) {
    var symbols = Object.getOwnPropertySymbols(object);

    if (enumerableOnly) {
      symbols = symbols.filter(function (sym) {
        return Object.getOwnPropertyDescriptor(object, sym).enumerable;
      });
    }

    keys.push.apply(keys, symbols);
  }

  return keys;
}

function _objectSpread2(target) {
  for (var i = 1; i < arguments.length; i++) {
    var source = arguments[i] != null ? arguments[i] : {};

    if (i % 2) {
      ownKeys(Object(source), true).forEach(function (key) {
        _defineProperty(target, key, source[key]);
      });
    } else if (Object.getOwnPropertyDescriptors) {
      Object.defineProperties(target, Object.getOwnPropertyDescriptors(source));
    } else {
      ownKeys(Object(source)).forEach(function (key) {
        Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key));
      });
    }
  }

  return target;
}

function _typeof(obj) {
  "@babel/helpers - typeof";

  if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") {
    _typeof = function (obj) {
      return typeof obj;
    };
  } else {
    _typeof = function (obj) {
      return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj;
    };
  }

  return _typeof(obj);
}

function _classCallCheck(instance, Constructor) {
  if (!(instance instanceof Constructor)) {
    throw new TypeError("Cannot call a class as a function");
  }
}

function _defineProperties(target, props) {
  for (var i = 0; i < props.length; i++) {
    var descriptor = props[i];
    descriptor.enumerable = descriptor.enumerable || false;
    descriptor.configurable = true;
    if ("value" in descriptor) descriptor.writable = true;
    Object.defineProperty(target, descriptor.key, descriptor);
  }
}

function _createClass(Constructor, protoProps, staticProps) {
  if (protoProps) _defineProperties(Constructor.prototype, protoProps);
  if (staticProps) _defineProperties(Constructor, staticProps);
  return Constructor;
}

function _defineProperty(obj, key, value) {
  if (key in obj) {
    Object.defineProperty(obj, key, {
      value: value,
      enumerable: true,
      configurable: true,
      writable: true
    });
  } else {
    obj[key] = value;
  }

  return obj;
}

function _slicedToArray(arr, i) {
  return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _unsupportedIterableToArray(arr, i) || _nonIterableRest();
}

function _arrayWithHoles(arr) {
  if (Array.isArray(arr)) return arr;
}

function _iterableToArrayLimit(arr, i) {
  var _i = arr == null ? null : typeof Symbol !== "undefined" && arr[Symbol.iterator] || arr["@@iterator"];

  if (_i == null) return;
  var _arr = [];
  var _n = true;
  var _d = false;

  var _s, _e;

  try {
    for (_i = _i.call(arr); !(_n = (_s = _i.next()).done); _n = true) {
      _arr.push(_s.value);

      if (i && _arr.length === i) break;
    }
  } catch (err) {
    _d = true;
    _e = err;
  } finally {
    try {
      if (!_n && _i["return"] != null) _i["return"]();
    } finally {
      if (_d) throw _e;
    }
  }

  return _arr;
}

function _unsupportedIterableToArray(o, minLen) {
  if (!o) return;
  if (typeof o === "string") return _arrayLikeToArray(o, minLen);
  var n = Object.prototype.toString.call(o).slice(8, -1);
  if (n === "Object" && o.constructor) n = o.constructor.name;
  if (n === "Map" || n === "Set") return Array.from(o);
  if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen);
}

function _arrayLikeToArray(arr, len) {
  if (len == null || len > arr.length) len = arr.length;

  for (var i = 0, arr2 = new Array(len); i < len; i++) arr2[i] = arr[i];

  return arr2;
}

function _nonIterableRest() {
  throw new TypeError("Invalid attempt to destructure non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.");
}

var StorageLimitType;

(function (StorageLimitType) {
  StorageLimitType["USER"] = "1";
  StorageLimitType["ORG"] = "2";
})(StorageLimitType || (StorageLimitType = {}));

var STORAGE_EXPIRED = 'expired';

var BlLocalStorage = /*#__PURE__*/function () {
  function BlLocalStorage() {
    var _this = this;

    _classCallCheck(this, BlLocalStorage);

    this.orgId = void 0;
    this.userId = void 0;

    this.setBaseInfo = function (userId, orgId) {
      _this.userId = userId;
      _this.orgId = orgId;
    };

    this.clearBaseInfo = function () {
      _this.userId = undefined;
      _this.orgId = undefined;
    };

    this.getLimitKey = function (limitType) {
      if (limitType === StorageLimitType.USER && _this.userId) {
        return "userId-".concat(_this.userId);
      }

      if (limitType === StorageLimitType.ORG && _this.orgId) {
        return "orgId-".concat(_this.orgId);
      }

      return '';
    };
  }

  _createClass(BlLocalStorage, [{
    key: "get",
    value: function get(key) {
      var localData = localStorage.getItem(key);

      if (localData) {
        var value = JSON.parse(localData);
        var now = new Date().getTime();
        var _value$meta = value.meta,
            storageTime = _value$meta.time,
            expiration = _value$meta.expiration,
            limitType = _value$meta.limitType;
        var isTimeIncompatible = expiration !== 0 && now - storageTime > expiration;

        if (limitType) {
          var strorageCurrentTime = _.get(value, "data[".concat(this.getLimitKey(limitType), "].time"));

          var isCurrentTimeIncompatible = expiration !== 0 && now - strorageCurrentTime > expiration;

          if (isCurrentTimeIncompatible) {
            this.remove(key);
            return STORAGE_EXPIRED;
          }

          return _.get(value, "data[".concat(this.getLimitKey(limitType), "].data"));
        } else if (isTimeIncompatible) {
          this.remove(key);
          return STORAGE_EXPIRED;
        }

        return value.data;
      }

      return undefined;
    }
  }, {
    key: "set",
    value: function set(key, data, option) {
      var _ref = option || {},
          _ref$expiration = _ref.expiration,
          expiration = _ref$expiration === void 0 ? 0 : _ref$expiration,
          limitType = _ref.limitType;

      var currentTime = new Date().getTime();
      var newValue = {
        meta: {
          expiration: expiration * 1000,
          time: currentTime,
          limitType: limitType
        },
        data: data
      };

      if (limitType === StorageLimitType.USER) {
        if (!this.userId) {
          console.error('设置用户限制类型的LocalStorage必须有用户信息');
          return;
        }

        newValue.data = _objectSpread2(_objectSpread2({}, this.getStorageData(key)), {}, _defineProperty({}, this.getLimitKey(limitType), {
          tiem: currentTime,
          data: data
        }));
      } else if (limitType === StorageLimitType.ORG) {
        if (!this.orgId) {
          console.error('设置工厂限制类型的LocalStorage必须有工厂信息');
          return;
        }

        newValue.data = _objectSpread2(_objectSpread2({}, this.getStorageData(key)), {}, _defineProperty({}, this.getLimitKey(limitType), {
          tiem: currentTime,
          data: data
        }));
      }

      localStorage.setItem(key, JSON.stringify(newValue));
    }
  }, {
    key: "remove",
    value: function remove(key) {
      var meta = this.getStorageMeta(key);

      if (meta) {
        var limitType = meta.limitType,
            expiration = meta.expiration;

        if (limitType) {
          this.set(key, undefined, {
            expiration: expiration,
            limitType: limitType
          });
        } else {
          localStorage.removeItem(key);
        }
      } else {
        localStorage.removeItem(key);
      }
    }
  }, {
    key: "clear",
    value: function clear() {
      localStorage.clear();
    }
  }, {
    key: "getStorageMeta",
    value: function getStorageMeta(key) {
      var storageJsonData = localStorage.getItem(key);

      if (storageJsonData) {
        var value = JSON.parse(storageJsonData);
        return value.meta;
      }

      return undefined;
    }
  }, {
    key: "getStorageData",
    value: function getStorageData(key) {
      var storageJsonData = localStorage.getItem(key);

      if (storageJsonData) {
        var value = JSON.parse(storageJsonData);
        return value.data;
      }

      return undefined;
    }
  }], [{
    key: "getInstance",
    value: function getInstance() {
      if (BlLocalStorage.instance === undefined) {
        BlLocalStorage.instance = new BlLocalStorage();
      }

      return BlLocalStorage.instance;
    }
  }]);

  return BlLocalStorage;
}();

BlLocalStorage.instance = void 0;

var qs = {
  parse: function parse(init) {
    return Object.fromEntries(new URLSearchParams(init !== null && init !== void 0 ? init : location.search));
  },
  stringify: function stringify(obj) {
    return new URLSearchParams(obj).toString();
  }
};
var BlUrl = {
  getParams: function getParams(init) {
    var query = qs.parse(init);
    return _.mapValues(query, function (value) {
      if (/^{.*}$/.test(value) || /^\[.*\]$/.test(value)) {
        return JSON.parse(value);
      }

      return value;
    });
  },
  setParams: function setParams(params) {
    var jsonStringifiedParams = Object.fromEntries(Object.entries(params).filter(function (_ref) {
      var _ref2 = _slicedToArray(_ref, 2),
          __ = _ref2[0],
          value = _ref2[1];

      return !_.isNil(value);
    }).map(function (_ref3) {
      var _ref4 = _slicedToArray(_ref3, 2),
          key = _ref4[0],
          value = _ref4[1];

      return [key, _typeof(value) === 'object' ? JSON.stringify(value) : value];
    }));
    return qs.stringify(jsonStringifiedParams);
  },
  replaceState: function replaceState(paramsStr) {
    window.history.replaceState(null, '', "".concat(location.pathname, "?").concat(paramsStr));
  }
};

var source = '0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz';
function randomString() {
  var len = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 20;
  var str = '';

  for (var i = 0; i < len; i++) {
    str += source[Math.floor(Math.random() * source.length)];
  }

  return str;
}

var FilterFieldType = {
  text: 1,
  number: 2,
  textArea: 3,
  select: 4,
  multiSelect: 5,
  boolean: 6,
  integer: 7,
  date: 8,
  url: 9,
  reference: 10,
  relation: 11,
  subordinate: 13,
  appendix: 14
};

export { BlLocalStorage, BlUrl, FilterFieldType, STORAGE_EXPIRED, StorageLimitType, qs, randomString };
